/* ========================================================================
 * Copyright (c) 2011 The University of Washington
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================================
 */

/*
   Sample client for gwsd.
  */

#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <ctype.h>
#include <strings.h>
#include <errno.h>
#include <pthread.h>
 
#include "gwsd.h"

extern int      errno;

#define ifdebug(l) if (debug>=l)
int    debug = 0;                       /* debug level */

char    *my_name;
int seq = 0;
 

/* main */
 
void usage() {
   fprintf(stderr, "usage: $s [-d <debug>] [-s gwsd_socket] -p password_file -u uwnetid -g group\n");
   exit (1);
}


int main(argc, argv)
int argc;
char **argv;
{
    int f;
    int i;
    int len;
    size_t fromlen;
    struct sockaddr_un sin, fromunix;
    int funix;
    int lfd;
    char pidt[12];
    int bedaemon = 1;
    char *sock_name = "/var/run/gwsd.sock";  // default
    char *userid = NULL;
    char *groupid = NULL;
    int seqno = 123;
    char *pwfile = NULL;
    char pw[65]; 
  
    my_name = argv[0];
    while (--argc > 0) {
       argv++;
       if (argv[0][0] == '-') {
          switch (argv[0][1]) {
          case 's':
             --argc;
             argv++;
             sock_name = argv[0];
             break;
          case 'p':
             --argc;
             argv++;
             pwfile = argv[0];
             break;
          case 'd':
             debug = argv[0][2] - '0';
             if ((debug<0) || (debug>9)) debug = 0;
             break;
          case 'u':
             --argc;
             argv++;
             userid = argv[0];
             break;
          case 'g':
             --argc;
             argv++;
             groupid = argv[0];
             break;
          }
        }
    }
 
    if (!(userid&&groupid&&pwfile)) usage();

   FILE *pf = fopen(pwfile, "r");
   if (pf && fgets(pw, 64, pf) && strlen(pw)>4) {
        char *s;
        if (s=strchr(pw, '\n')) *s = '\0';
    } else {
        perror(pwfile);
        exit (1);
    }

    // connect to gwsd
    
   printf("connecting\n");
   struct sockaddr_un sun;
   int gwsd_sock;

   sun.sun_family = AF_UNIX;
   strcpy(sun.sun_path, sock_name);
   if ((gwsd_sock=socket(AF_UNIX, SOCK_STREAM, 0))>0 &&
      connect(gwsd_sock, (struct sockaddr *)&sun, strlen(sun.sun_path)+2)==0) {
   } else if (gwsd_sock) {
      perror("gws_gwsd_connect");
      exit (1);
   }

   // login 

   ClientMsg_ msg;
   ResponseMsg_ rsp;

   printf("authenticating\n");
   msg.cmd = CMD_PASSWORD;
   msg.seq = seqno++;
   strncpy((char*)msg.id, pw, MSG_ID_LEN);
   int nb = write(gwsd_sock, (char*)&msg, sizeof(ClientMsg_));
   if (nb!=sizeof(ClientMsg_)) {
     perror("gws_gwsd_write");
     exit (1);
   }
    
   nb = read(gwsd_sock, &rsp, sizeof(ResponseMsg_));
   if (nb!=sizeof(ResponseMsg_)) {
     perror("gws_gwsd_read");
     exit (1);
   }
   if (rsp.seq!=msg.seq)  {
     fprintf(stderr, "gws_gwsd_seq was %d, expected %d", rsp.seq, msg.seq);
     exit (1);
   }
   printf("status: %d\n",  rsp.resp);

   printf("checking membership\n");
   msg.cmd = CMD_MEMBER_OF;
   msg.seq = seqno++;
   strncpy((char*)msg.id, userid, MSG_ID_LEN);
   strncpy((char*)msg.group, groupid, MSG_GROUP_LEN);
   nb = write(gwsd_sock, (char*)&msg, sizeof(ClientMsg_));
   if (nb!=sizeof(ClientMsg_)) {
     perror("gws_gwsd_write");
     exit (1);
   }
    
   nb = read(gwsd_sock, &rsp, sizeof(ResponseMsg_));
   if (nb!=sizeof(ResponseMsg_)) {
     perror("gws_gwsd_read");
     exit (1);
   }
   if (rsp.seq!=msg.seq)  {
     fprintf(stderr, "gws_gwsd_seq was %d, expected %d", rsp.seq, msg.seq);
     exit (1);
   }
   printf("status: %d\n",  rsp.resp);
   exit (0);
}
